@@ -15,8 +15,9 @@ module Text.Highlighting.Kate.Common where
 #ifdef _PCRE_LIGHT
 import Text.Regex.PCRE.Light.Char8
 #else
+import Data.ByteString.UTF8 (fromString, toString)
 import System.IO.Unsafe (unsafePerformIO)
-import Text.Regex.PCRE.String
+import Text.Regex.PCRE.ByteString
 #endif
 import Text.Highlighting.Kate.Types
 import Text.ParserCombinators.Parsec hiding (State)
@@ -164,52 +165,67 @@ subDynamic ('%':x:xs) | isDigit x = do
 subDynamic (x:xs) = subDynamic xs >>= return . (x:)
 subDynamic "" = return ""
 
+-- convert octal escapes to the form pcre wants.  Note:
+-- need at least pcre 8.34 for the form \o{dddd}.
+-- So we prefer \ddd when possible.
 convertOctal :: String -> String
 convertOctal [] = ""
-convertOctal ('\\':'0':x:y:z:rest) | isOctalDigit x && isOctalDigit y && isOctalDigit z =
-  case reads ['\\','o',x,y,z] of
-        ((x,rest):_) -> x : convertOctal rest
-        _            -> '\\':'o':x:y:z: convertOctal rest
-convertOctal ('\\':x:y:z:rest) | isOctalDigit x && isOctalDigit y && isOctalDigit z =
-  case reads ['\\','o',x,y,z] of
-        ((x,rest):_) -> x : convertOctal rest
-        _            -> '\\':'o':x:y:z: convertOctal rest
+convertOctal ('\\':'0':x:y:z:rest)
+  | all isOctalDigit [x,y,z] = '\\':x:y:z: convertOctal rest
+convertOctal ('\\':x:y:z:rest)
+  | all isOctalDigit [x,y,z] ='\\':x:y:z: convertOctal rest
+convertOctal ('\\':'o':'{':zs) =
+  case break (=='}') zs of
+       (ds, '}':rest) | all isOctalDigit ds && not (null ds) ->
+         if length ds <= 3
+            then '\\': pad3 ds ++ convertOctal rest
+            else case reads ('\\':'o':ds) of
+                   ((c,[]):_) -> c : convertOctal rest
+                   _          -> '\\':'o':'{': convertOctal zs
+       _  -> '\\':'o':'{': convertOctal zs
+    where pad3 [x]   = ['0','0',x]
+          pad3 [x,y] = ['0',x,y]
+          pad3 xs    = xs
 convertOctal (x:xs) = x : convertOctal xs
 
 isOctalDigit :: Char -> Bool
 isOctalDigit c = c == '0' || c == '1' || c == '2' || c == '3'
               || c == '4' || c == '5' || c == '6' || c == '7'
 
-compileRegex :: String -> KateParser Regex
+compileRegex :: Bool -> String -> Regex
+compileRegex caseSensitive regexpStr =
 #ifdef _PCRE_LIGHT
-compileRegex regexpStr = return $ compile ('.' : convertOctal regexpStr) [anchored]
+  let opts = [anchored, utf8] ++ [caseless | not caseSensitive]
+  in  compile ('.' : convertOctal regexpStr) opts
 #else
-compileRegex regexpStr =
-  case unsafePerformIO $ compile (compAnchored) (execNotEmpty)
-       ('.' : convertOctal regexpStr) of
-        Left _  -> fail $ "Error compiling regex: " ++ show regexpStr
-        Right r -> return r
+  let opts = compAnchored + compUTF8 +
+               if caseSensitive then 0 else compCaseless
+  in  case unsafePerformIO $ compile opts (execNotEmpty)
+           (fromString ('.' : convertOctal regexpStr)) of
+            Left e  -> error $ "Error compiling regex: " ++ show regexpStr ++
+                               "\n" ++ show e
+            Right r -> r
 #endif
 
 matchRegex :: Regex -> String -> KateParser (Maybe [String])
 #ifdef _PCRE_LIGHT
 matchRegex r s = return $ match r s [exec_notempty]
 #else
-matchRegex r s = case unsafePerformIO (regexec r s) of
-                      Right (Just (_, mat, _ , capts)) -> return $ Just (mat : capts)
+matchRegex r s = case unsafePerformIO (regexec r (fromString s)) of
+                      Right (Just (_, mat, _ , capts)) -> return $
+                                       Just $ map toString (mat : capts)
                       Right Nothing -> return Nothing
                       Left matchError -> fail $ show matchError
 #endif
 
-pRegExpr :: KateParser Regex -> KateParser String
-pRegExpr compiledRegex = do
+pRegExpr :: Regex -> KateParser String
+pRegExpr regex = do
   rest <- getInput
   prevChar <- fromState synStPrevChar
   -- Note: we keep one preceding character, so initial \b can match or not...
   let target = if prevChar == '\n'
                   then ' ':rest
                   else prevChar:rest
-  regex <- compiledRegex
   matches <- matchRegex regex target
   case matches of
         Just (x:xs) | null x -> fail "Regex matched null string!"
@@ -222,28 +238,30 @@ pRegExpr compiledRegex = do
 pRegExprDynamic :: [Char] -> KateParser String
 pRegExprDynamic regexpStr = do
   regexpStr' <- subDynamic regexpStr
-  pRegExpr $ compileRegex regexpStr'
+  caseSensitive <- synStCaseSensitive `fmap` getState
+  pRegExpr $ compileRegex caseSensitive regexpStr'
 
-integerRegex :: KateParser Regex
-integerRegex = compileRegex "\\b[-+]?(0[Xx][0-9A-Fa-f]+|0[Oo][0-7]+|[0-9]+)\\b"
+integerRegex :: Regex
+integerRegex =
+  compileRegex True "\\b[-+]?(0[Xx][0-9A-Fa-f]+|0[Oo][0-7]+|[0-9]+)\\b"
 
 pInt :: KateParser String
 pInt = pRegExpr integerRegex
 
-floatRegex :: KateParser Regex
-floatRegex = compileRegex "\\b[-+]?(([0-9]+\\.[0-9]*|[0-9]*\\.[0-9]+)([Ee][-+]?[0-9]+)?|[0-9]+[Ee][-+]?[0-9]+)\\b"
+floatRegex :: Regex
+floatRegex = compileRegex True "\\b[-+]?(([0-9]+\\.[0-9]*|[0-9]*\\.[0-9]+)([Ee][-+]?[0-9]+)?|[0-9]+[Ee][-+]?[0-9]+)\\b"
 
 pFloat :: KateParser String
 pFloat = pRegExpr floatRegex
 
-octRegex :: KateParser Regex
-octRegex = compileRegex "\\b[-+]?0[Oo][0-7]+\\b"
+octRegex :: Regex
+octRegex = compileRegex True "\\b[-+]?0[Oo][0-7]+\\b"
 
 pHlCOct :: KateParser String
 pHlCOct = pRegExpr octRegex
 
-hexRegex :: KateParser Regex
-hexRegex = compileRegex "\\b[-+]?0[Xx][0-9A-Fa-f]+\\b"
+hexRegex :: Regex
+hexRegex = compileRegex True "\\b[-+]?0[Xx][0-9A-Fa-f]+\\b"
 
 pHlCHex :: KateParser String
 pHlCHex = pRegExpr hexRegex
@@ -278,10 +296,12 @@ pLineContinue = try $ char '\\' >> eof >> return "\\"
 pDetectSpaces :: KateParser [Char]
 pDetectSpaces = many1 (satisfy $ \c -> c == ' ' || c == '\t')
 
+-- http://docs.kde.org/stable/en/applications/kate/kate-highlight-rules-detailled.html says this is
+-- [a-zA-Z_][a-zA-Z0-9_]*
 pDetectIdentifier :: KateParser [Char]
 pDetectIdentifier = do
-  first <- letter
-  rest <- many alphaNum
+  first <- letter <|> char '_'
+  rest <- many (alphaNum <|> char '_')
   return (first:rest)
 
 fromState :: (SyntaxState -> a) -> KateParser a
